@@ -152,7 +152,7 @@
 
 
 /* Copy the first part of user declarations.  */
-#line 37 "ada-exp.y"
+#line 36 "ada-exp.y"
 
 
 #include "defs.h"
@@ -249,10 +249,10 @@ static struct stoken string_to_operator (struct stoken);
 
 static void write_int (LONGEST, struct type *);
 
-static void write_object_renaming (struct block *, const char *, int,
+static void write_object_renaming (const struct block *, const char *, int,
 				   const char *, int);
 
-static struct type* write_var_or_type (struct block *, struct stoken);
+static struct type* write_var_or_type (const struct block *, struct stoken);
 
 static void write_name_assoc (struct stoken);
 
@@ -262,7 +262,7 @@ static struct block *block_lookup (struct block *, char *);
 
 static LONGEST convert_char_literal (struct type *, LONGEST);
 
-static void write_ambiguous_var (struct block *, char *, int);
+static void write_ambiguous_var (const struct block *, char *, int);
 
 static struct type *type_int (void);
 
@@ -298,7 +298,7 @@ static struct type *type_system_address (void);
 #endif
 
 #if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
-#line 169 "ada-exp.y"
+#line 168 "ada-exp.y"
 typedef union YYSTYPE {
     LONGEST lval;
     struct {
@@ -559,19 +559,19 @@ static const yysigned_char yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const unsigned short yyrline[] =
 {
-       0,   234,   234,   238,   239,   241,   246,   250,   254,   260,
-     279,   279,   291,   295,   297,   305,   316,   326,   330,   333,
-     336,   340,   344,   348,   352,   355,   357,   359,   361,   365,
-     378,   382,   386,   390,   394,   398,   402,   406,   410,   414,
-     417,   421,   425,   429,   431,   436,   444,   448,   454,   465,
-     469,   473,   477,   478,   479,   480,   481,   482,   486,   488,
-     493,   495,   500,   502,   507,   509,   513,   515,   527,   529,
-     535,   538,   541,   544,   546,   548,   550,   552,   554,   556,
-     560,   562,   567,   577,   579,   585,   589,   596,   604,   608,
-     614,   616,   620,   624,   626,   628,   636,   647,   649,   654,
-     663,   664,   670,   675,   681,   690,   691,   692,   696,   701,
-     716,   715,   718,   721,   720,   726,   725,   728,   731,   730,
-     738,   740,   742
+       0,   233,   233,   237,   238,   240,   245,   249,   253,   259,
+     278,   278,   290,   294,   296,   304,   315,   325,   329,   332,
+     335,   339,   343,   347,   351,   354,   356,   358,   360,   364,
+     377,   381,   385,   389,   393,   397,   401,   405,   409,   413,
+     416,   420,   424,   428,   430,   435,   443,   447,   453,   464,
+     468,   472,   476,   477,   478,   479,   480,   481,   485,   487,
+     492,   494,   499,   501,   506,   508,   512,   514,   526,   528,
+     534,   537,   540,   543,   545,   547,   549,   551,   553,   555,
+     559,   561,   566,   576,   578,   584,   588,   595,   603,   607,
+     613,   615,   619,   623,   625,   627,   635,   646,   648,   653,
+     662,   663,   669,   674,   680,   689,   690,   691,   695,   700,
+     715,   714,   717,   720,   719,   725,   724,   727,   730,   729,
+     737,   739,   741
 };
 #endif
 
@@ -1537,27 +1537,27 @@ yyreduce:
   switch (yyn)
     {
         case 4:
-#line 240 "ada-exp.y"
+#line 239 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_COMMA); }
     break;
 
   case 5:
-#line 242 "ada-exp.y"
+#line 241 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_ASSIGN); }
     break;
 
   case 6:
-#line 247 "ada-exp.y"
+#line 246 "ada-exp.y"
     { write_exp_elt_opcode (UNOP_IND); }
     break;
 
   case 7:
-#line 251 "ada-exp.y"
+#line 250 "ada-exp.y"
     { write_exp_op_with_string (STRUCTOP_STRUCT, yyvsp[0].sval); }
     break;
 
   case 8:
-#line 255 "ada-exp.y"
+#line 254 "ada-exp.y"
     {
 			  write_exp_elt_opcode (OP_FUNCALL);
 			  write_exp_elt_longcst (yyvsp[-1].lval);
@@ -1566,7 +1566,7 @@ yyreduce:
     break;
 
   case 9:
-#line 261 "ada-exp.y"
+#line 260 "ada-exp.y"
     {
 			  if (yyvsp[-3].tval != NULL)
 			    {
@@ -1586,12 +1586,12 @@ yyreduce:
     break;
 
   case 10:
-#line 279 "ada-exp.y"
+#line 278 "ada-exp.y"
     { type_qualifier = yyvsp[-2].tval; }
     break;
 
   case 11:
-#line 281 "ada-exp.y"
+#line 280 "ada-exp.y"
     {
 			  if (yyvsp[-6].tval == NULL)
 			    error (_("Type required for qualification"));
@@ -1603,17 +1603,17 @@ yyreduce:
     break;
 
   case 12:
-#line 291 "ada-exp.y"
+#line 290 "ada-exp.y"
     { yyval.tval = type_qualifier; }
     break;
 
   case 13:
-#line 296 "ada-exp.y"
+#line 295 "ada-exp.y"
     { write_exp_elt_opcode (TERNOP_SLICE); }
     break;
 
   case 14:
-#line 298 "ada-exp.y"
+#line 297 "ada-exp.y"
     { if (yyvsp[-5].tval == NULL) 
                             write_exp_elt_opcode (TERNOP_SLICE);
 			  else
@@ -1622,12 +1622,12 @@ yyreduce:
     break;
 
   case 15:
-#line 305 "ada-exp.y"
+#line 304 "ada-exp.y"
     { }
     break;
 
   case 16:
-#line 317 "ada-exp.y"
+#line 316 "ada-exp.y"
     { if (yyvsp[0].tval != NULL)
 			    {
 			      write_exp_elt_opcode (OP_TYPE);
@@ -1638,57 +1638,57 @@ yyreduce:
     break;
 
   case 17:
-#line 327 "ada-exp.y"
+#line 326 "ada-exp.y"
     { write_dollar_variable (yyvsp[0].sval); }
     break;
 
   case 20:
-#line 337 "ada-exp.y"
+#line 336 "ada-exp.y"
     { write_exp_elt_opcode (UNOP_NEG); }
     break;
 
   case 21:
-#line 341 "ada-exp.y"
+#line 340 "ada-exp.y"
     { write_exp_elt_opcode (UNOP_PLUS); }
     break;
 
   case 22:
-#line 345 "ada-exp.y"
+#line 344 "ada-exp.y"
     { write_exp_elt_opcode (UNOP_LOGICAL_NOT); }
     break;
 
   case 23:
-#line 349 "ada-exp.y"
+#line 348 "ada-exp.y"
     { write_exp_elt_opcode (UNOP_ABS); }
     break;
 
   case 24:
-#line 352 "ada-exp.y"
+#line 351 "ada-exp.y"
     { yyval.lval = 0; }
     break;
 
   case 25:
-#line 356 "ada-exp.y"
+#line 355 "ada-exp.y"
     { yyval.lval = 1; }
     break;
 
   case 26:
-#line 358 "ada-exp.y"
+#line 357 "ada-exp.y"
     { yyval.lval = 1; }
     break;
 
   case 27:
-#line 360 "ada-exp.y"
+#line 359 "ada-exp.y"
     { yyval.lval = yyvsp[-2].lval + 1; }
     break;
 
   case 28:
-#line 362 "ada-exp.y"
+#line 361 "ada-exp.y"
     { yyval.lval = yyvsp[-4].lval + 1; }
     break;
 
   case 29:
-#line 367 "ada-exp.y"
+#line 366 "ada-exp.y"
     { 
 			  if (yyvsp[-2].tval == NULL)
 			    error (_("Type required within braces in coercion"));
@@ -1699,72 +1699,72 @@ yyreduce:
     break;
 
   case 30:
-#line 379 "ada-exp.y"
+#line 378 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_EXP); }
     break;
 
   case 31:
-#line 383 "ada-exp.y"
+#line 382 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_MUL); }
     break;
 
   case 32:
-#line 387 "ada-exp.y"
+#line 386 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_DIV); }
     break;
 
   case 33:
-#line 391 "ada-exp.y"
+#line 390 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_REM); }
     break;
 
   case 34:
-#line 395 "ada-exp.y"
+#line 394 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_MOD); }
     break;
 
   case 35:
-#line 399 "ada-exp.y"
+#line 398 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_REPEAT); }
     break;
 
   case 36:
-#line 403 "ada-exp.y"
+#line 402 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_ADD); }
     break;
 
   case 37:
-#line 407 "ada-exp.y"
+#line 406 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_CONCAT); }
     break;
 
   case 38:
-#line 411 "ada-exp.y"
+#line 410 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_SUB); }
     break;
 
   case 40:
-#line 418 "ada-exp.y"
+#line 417 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_EQUAL); }
     break;
 
   case 41:
-#line 422 "ada-exp.y"
+#line 421 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_NOTEQUAL); }
     break;
 
   case 42:
-#line 426 "ada-exp.y"
+#line 425 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_LEQ); }
     break;
 
   case 43:
-#line 430 "ada-exp.y"
+#line 429 "ada-exp.y"
     { write_exp_elt_opcode (TERNOP_IN_RANGE); }
     break;
 
   case 44:
-#line 432 "ada-exp.y"
+#line 431 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_IN_BOUNDS);
 			  write_exp_elt_longcst ((LONGEST) yyvsp[0].lval);
 			  write_exp_elt_opcode (BINOP_IN_BOUNDS);
@@ -1772,7 +1772,7 @@ yyreduce:
     break;
 
   case 45:
-#line 437 "ada-exp.y"
+#line 436 "ada-exp.y"
     { 
 			  if (yyvsp[0].tval == NULL)
 			    error (_("Right operand of 'in' must be type"));
@@ -1783,14 +1783,14 @@ yyreduce:
     break;
 
   case 46:
-#line 445 "ada-exp.y"
+#line 444 "ada-exp.y"
     { write_exp_elt_opcode (TERNOP_IN_RANGE);
 		          write_exp_elt_opcode (UNOP_LOGICAL_NOT);
 			}
     break;
 
   case 47:
-#line 449 "ada-exp.y"
+#line 448 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_IN_BOUNDS);
 			  write_exp_elt_longcst ((LONGEST) yyvsp[0].lval);
 			  write_exp_elt_opcode (BINOP_IN_BOUNDS);
@@ -1799,7 +1799,7 @@ yyreduce:
     break;
 
   case 48:
-#line 455 "ada-exp.y"
+#line 454 "ada-exp.y"
     { 
 			  if (yyvsp[0].tval == NULL)
 			    error (_("Right operand of 'in' must be type"));
@@ -1811,77 +1811,77 @@ yyreduce:
     break;
 
   case 49:
-#line 466 "ada-exp.y"
+#line 465 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_GEQ); }
     break;
 
   case 50:
-#line 470 "ada-exp.y"
+#line 469 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_LESS); }
     break;
 
   case 51:
-#line 474 "ada-exp.y"
+#line 473 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_GTR); }
     break;
 
   case 58:
-#line 487 "ada-exp.y"
+#line 486 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_BITWISE_AND); }
     break;
 
   case 59:
-#line 489 "ada-exp.y"
+#line 488 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_BITWISE_AND); }
     break;
 
   case 60:
-#line 494 "ada-exp.y"
+#line 493 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_LOGICAL_AND); }
     break;
 
   case 61:
-#line 496 "ada-exp.y"
+#line 495 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_LOGICAL_AND); }
     break;
 
   case 62:
-#line 501 "ada-exp.y"
+#line 500 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_BITWISE_IOR); }
     break;
 
   case 63:
-#line 503 "ada-exp.y"
+#line 502 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_BITWISE_IOR); }
     break;
 
   case 64:
-#line 508 "ada-exp.y"
+#line 507 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_LOGICAL_OR); }
     break;
 
   case 65:
-#line 510 "ada-exp.y"
+#line 509 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_LOGICAL_OR); }
     break;
 
   case 66:
-#line 514 "ada-exp.y"
+#line 513 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_BITWISE_XOR); }
     break;
 
   case 67:
-#line 516 "ada-exp.y"
+#line 515 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_BITWISE_XOR); }
     break;
 
   case 68:
-#line 528 "ada-exp.y"
+#line 527 "ada-exp.y"
     { write_exp_elt_opcode (UNOP_ADDR); }
     break;
 
   case 69:
-#line 530 "ada-exp.y"
+#line 529 "ada-exp.y"
     { write_exp_elt_opcode (UNOP_ADDR);
 			  write_exp_elt_opcode (UNOP_CAST);
 			  write_exp_elt_type (type_system_address ());
@@ -1890,70 +1890,70 @@ yyreduce:
     break;
 
   case 70:
-#line 536 "ada-exp.y"
+#line 535 "ada-exp.y"
     { write_int (yyvsp[0].lval, type_int ());
 			  write_exp_elt_opcode (OP_ATR_FIRST); }
     break;
 
   case 71:
-#line 539 "ada-exp.y"
+#line 538 "ada-exp.y"
     { write_int (yyvsp[0].lval, type_int ());
 			  write_exp_elt_opcode (OP_ATR_LAST); }
     break;
 
   case 72:
-#line 542 "ada-exp.y"
+#line 541 "ada-exp.y"
     { write_int (yyvsp[0].lval, type_int ());
 			  write_exp_elt_opcode (OP_ATR_LENGTH); }
     break;
 
   case 73:
-#line 545 "ada-exp.y"
+#line 544 "ada-exp.y"
     { write_exp_elt_opcode (OP_ATR_SIZE); }
     break;
 
   case 74:
-#line 547 "ada-exp.y"
+#line 546 "ada-exp.y"
     { write_exp_elt_opcode (OP_ATR_TAG); }
     break;
 
   case 75:
-#line 549 "ada-exp.y"
+#line 548 "ada-exp.y"
     { write_exp_elt_opcode (OP_ATR_MIN); }
     break;
 
   case 76:
-#line 551 "ada-exp.y"
+#line 550 "ada-exp.y"
     { write_exp_elt_opcode (OP_ATR_MAX); }
     break;
 
   case 77:
-#line 553 "ada-exp.y"
+#line 552 "ada-exp.y"
     { write_exp_elt_opcode (OP_ATR_POS); }
     break;
 
   case 78:
-#line 555 "ada-exp.y"
+#line 554 "ada-exp.y"
     { write_exp_elt_opcode (OP_ATR_VAL); }
     break;
 
   case 79:
-#line 557 "ada-exp.y"
+#line 556 "ada-exp.y"
     { write_exp_elt_opcode (OP_ATR_MODULUS); }
     break;
 
   case 80:
-#line 561 "ada-exp.y"
+#line 560 "ada-exp.y"
     { yyval.lval = 1; }
     break;
 
   case 81:
-#line 563 "ada-exp.y"
+#line 562 "ada-exp.y"
     { yyval.lval = yyvsp[-1].typed_val.val; }
     break;
 
   case 82:
-#line 568 "ada-exp.y"
+#line 567 "ada-exp.y"
     { 
 			  if (yyvsp[0].tval == NULL)
 			    error (_("Prefix must be type"));
@@ -1963,19 +1963,19 @@ yyreduce:
     break;
 
   case 84:
-#line 579 "ada-exp.y"
+#line 578 "ada-exp.y"
     { write_exp_elt_opcode (OP_TYPE);
 			  write_exp_elt_type (parse_type->builtin_void);
 			  write_exp_elt_opcode (OP_TYPE); }
     break;
 
   case 85:
-#line 586 "ada-exp.y"
+#line 585 "ada-exp.y"
     { write_int ((LONGEST) yyvsp[0].typed_val.val, yyvsp[0].typed_val.type); }
     break;
 
   case 86:
-#line 590 "ada-exp.y"
+#line 589 "ada-exp.y"
     { write_int (convert_char_literal (type_qualifier, yyvsp[0].typed_val.val),
 			       (type_qualifier == NULL) 
 			       ? yyvsp[0].typed_val.type : type_qualifier);
@@ -1983,7 +1983,7 @@ yyreduce:
     break;
 
   case 87:
-#line 597 "ada-exp.y"
+#line 596 "ada-exp.y"
     { write_exp_elt_opcode (OP_DOUBLE);
 			  write_exp_elt_type (yyvsp[0].typed_val_float.type);
 			  write_exp_elt_dblcst (yyvsp[0].typed_val_float.dval);
@@ -1992,44 +1992,44 @@ yyreduce:
     break;
 
   case 88:
-#line 605 "ada-exp.y"
+#line 604 "ada-exp.y"
     { write_int (0, type_int ()); }
     break;
 
   case 89:
-#line 609 "ada-exp.y"
+#line 608 "ada-exp.y"
     { 
 			  write_exp_op_with_string (OP_STRING, yyvsp[0].sval);
 			}
     break;
 
   case 90:
-#line 615 "ada-exp.y"
+#line 614 "ada-exp.y"
     { write_int (1, type_boolean ()); }
     break;
 
   case 91:
-#line 617 "ada-exp.y"
+#line 616 "ada-exp.y"
     { write_int (0, type_boolean ()); }
     break;
 
   case 92:
-#line 621 "ada-exp.y"
+#line 620 "ada-exp.y"
     { error (_("NEW not implemented.")); }
     break;
 
   case 93:
-#line 625 "ada-exp.y"
+#line 624 "ada-exp.y"
     { yyval.tval = write_var_or_type (NULL, yyvsp[0].sval); }
     break;
 
   case 94:
-#line 627 "ada-exp.y"
+#line 626 "ada-exp.y"
     { yyval.tval = write_var_or_type (yyvsp[-1].bval, yyvsp[0].sval); }
     break;
 
   case 95:
-#line 629 "ada-exp.y"
+#line 628 "ada-exp.y"
     { 
 			  yyval.tval = write_var_or_type (NULL, yyvsp[-1].sval);
 			  if (yyval.tval == NULL)
@@ -2040,7 +2040,7 @@ yyreduce:
     break;
 
   case 96:
-#line 637 "ada-exp.y"
+#line 636 "ada-exp.y"
     { 
 			  yyval.tval = write_var_or_type (yyvsp[-2].bval, yyvsp[-1].sval);
 			  if (yyval.tval == NULL)
@@ -2051,17 +2051,17 @@ yyreduce:
     break;
 
   case 97:
-#line 648 "ada-exp.y"
+#line 647 "ada-exp.y"
     { yyval.bval = block_lookup (NULL, yyvsp[-1].sval.ptr); }
     break;
 
   case 98:
-#line 650 "ada-exp.y"
+#line 649 "ada-exp.y"
     { yyval.bval = block_lookup (yyvsp[-2].bval, yyvsp[-1].sval.ptr); }
     break;
 
   case 99:
-#line 655 "ada-exp.y"
+#line 654 "ada-exp.y"
     {
 			  write_exp_elt_opcode (OP_AGGREGATE);
 			  write_exp_elt_longcst (yyvsp[-1].lval);
@@ -2070,12 +2070,12 @@ yyreduce:
     break;
 
   case 100:
-#line 663 "ada-exp.y"
+#line 662 "ada-exp.y"
     { yyval.lval = yyvsp[0].lval; }
     break;
 
   case 101:
-#line 665 "ada-exp.y"
+#line 664 "ada-exp.y"
     { write_exp_elt_opcode (OP_POSITIONAL);
 			  write_exp_elt_longcst (yyvsp[-1].lval);
 			  write_exp_elt_opcode (OP_POSITIONAL);
@@ -2084,12 +2084,12 @@ yyreduce:
     break;
 
   case 102:
-#line 671 "ada-exp.y"
+#line 670 "ada-exp.y"
     { yyval.lval = yyvsp[-1].lval + yyvsp[0].lval; }
     break;
 
   case 103:
-#line 676 "ada-exp.y"
+#line 675 "ada-exp.y"
     { write_exp_elt_opcode (OP_POSITIONAL);
 			  write_exp_elt_longcst (0);
 			  write_exp_elt_opcode (OP_POSITIONAL);
@@ -2098,7 +2098,7 @@ yyreduce:
     break;
 
   case 104:
-#line 682 "ada-exp.y"
+#line 681 "ada-exp.y"
     { write_exp_elt_opcode (OP_POSITIONAL);
 			  write_exp_elt_longcst (yyvsp[-2].lval);
 			  write_exp_elt_opcode (OP_POSITIONAL);
@@ -2107,27 +2107,27 @@ yyreduce:
     break;
 
   case 105:
-#line 690 "ada-exp.y"
+#line 689 "ada-exp.y"
     { yyval.lval = 1; }
     break;
 
   case 106:
-#line 691 "ada-exp.y"
+#line 690 "ada-exp.y"
     { yyval.lval = 1; }
     break;
 
   case 107:
-#line 693 "ada-exp.y"
+#line 692 "ada-exp.y"
     { yyval.lval = yyvsp[0].lval + 1; }
     break;
 
   case 108:
-#line 697 "ada-exp.y"
+#line 696 "ada-exp.y"
     { write_exp_elt_opcode (OP_OTHERS); }
     break;
 
   case 109:
-#line 702 "ada-exp.y"
+#line 701 "ada-exp.y"
     {
 			  write_exp_elt_opcode (OP_CHOICES);
 			  write_exp_elt_longcst (yyvsp[0].lval);
@@ -2136,69 +2136,69 @@ yyreduce:
     break;
 
   case 110:
-#line 716 "ada-exp.y"
+#line 715 "ada-exp.y"
     { write_name_assoc (yyvsp[-1].sval); }
     break;
 
   case 111:
-#line 717 "ada-exp.y"
+#line 716 "ada-exp.y"
     { yyval.lval = 1; }
     break;
 
   case 112:
-#line 719 "ada-exp.y"
+#line 718 "ada-exp.y"
     { yyval.lval = 1; }
     break;
 
   case 113:
-#line 721 "ada-exp.y"
+#line 720 "ada-exp.y"
     { write_exp_elt_opcode (OP_DISCRETE_RANGE);
 			  write_exp_op_with_string (OP_NAME, empty_stoken);
 			}
     break;
 
   case 114:
-#line 724 "ada-exp.y"
+#line 723 "ada-exp.y"
     { yyval.lval = 1; }
     break;
 
   case 115:
-#line 726 "ada-exp.y"
+#line 725 "ada-exp.y"
     { write_name_assoc (yyvsp[-1].sval); }
     break;
 
   case 116:
-#line 727 "ada-exp.y"
+#line 726 "ada-exp.y"
     { yyval.lval = yyvsp[0].lval + 1; }
     break;
 
   case 117:
-#line 729 "ada-exp.y"
+#line 728 "ada-exp.y"
     { yyval.lval = yyvsp[0].lval + 1; }
     break;
 
   case 118:
-#line 731 "ada-exp.y"
+#line 730 "ada-exp.y"
     { write_exp_elt_opcode (OP_DISCRETE_RANGE); }
     break;
 
   case 119:
-#line 732 "ada-exp.y"
+#line 731 "ada-exp.y"
     { yyval.lval = yyvsp[0].lval + 1; }
     break;
 
   case 120:
-#line 739 "ada-exp.y"
+#line 738 "ada-exp.y"
     { write_exp_elt_opcode (UNOP_IND); }
     break;
 
   case 121:
-#line 741 "ada-exp.y"
+#line 740 "ada-exp.y"
     { write_exp_elt_opcode (UNOP_ADDR); }
     break;
 
   case 122:
-#line 743 "ada-exp.y"
+#line 742 "ada-exp.y"
     { write_exp_elt_opcode (BINOP_SUBSCRIPT); }
     break;
 
@@ -2431,7 +2431,7 @@ yyreturn:
 }
 
 
-#line 746 "ada-exp.y"
+#line 745 "ada-exp.y"
 
 
 /* yylex defined in ada-lex.c: Reads one token, getting characters */
@@ -2507,8 +2507,8 @@ string_to_operator (struct stoken string)
 /* Emit expression to access an instance of SYM, in block BLOCK (if
  * non-NULL), and with :: qualification ORIG_LEFT_CONTEXT.  */
 static void
-write_var_from_sym (struct block *orig_left_context,
-		    struct block *block,
+write_var_from_sym (const struct block *orig_left_context,
+		    const struct block *block,
 		    struct symbol *sym)
 {
   if (orig_left_context == NULL && symbol_read_needs_frame (sym))
@@ -2556,7 +2556,7 @@ write_exp_op_with_string (enum exp_opcode opcode, struct stoken token)
  * new encoding entirely (FIXME pnh 7/20/2007).  */
 
 static void
-write_object_renaming (struct block *orig_left_context,
+write_object_renaming (const struct block *orig_left_context,
 		       const char *renamed_entity, int renamed_entity_len,
 		       const char *renaming_expr, int max_depth)
 {
@@ -2570,7 +2570,7 @@ write_object_renaming (struct block *orig_left_context,
   if (orig_left_context == NULL)
     orig_left_context = get_selected_block (NULL);
 
-  name = obsavestring (renamed_entity, renamed_entity_len, &temp_parse_space);
+  name = obstack_copy0 (&temp_parse_space, renamed_entity, renamed_entity_len);
   ada_lookup_encoded_symbol (name, orig_left_context, VAR_DOMAIN, &sym_info);
   if (sym_info.sym == NULL)
     error (_("Could not find renamed variable: %s"), ada_decode (name));
@@ -2639,8 +2639,8 @@ write_object_renaming (struct block *orig_left_context,
 	      end = renaming_expr + strlen (renaming_expr);
 
 	    index_name =
-	      obsavestring (renaming_expr, end - renaming_expr,
-			    &temp_parse_space);
+	      obstack_copy0 (&temp_parse_space, renaming_expr,
+			     end - renaming_expr);
 	    renaming_expr = end;
 
 	    ada_lookup_encoded_symbol (index_name, NULL, VAR_DOMAIN,
@@ -2715,7 +2715,7 @@ block_lookup (struct block *context, char *raw_name)
   else
     name = ada_encode (raw_name);
 
-  nsyms = ada_lookup_symbol_list (name, context, VAR_DOMAIN, &syms, 1);
+  nsyms = ada_lookup_symbol_list (name, context, VAR_DOMAIN, &syms);
   if (context == NULL
       && (nsyms == 0 || SYMBOL_CLASS (syms[0].sym) != LOC_BLOCK))
     symtab = lookup_symtab (name);
@@ -2850,13 +2850,13 @@ write_selectors (char *sels)
    a temporary symbol that is valid until the next call to ada_parse.
    */
 static void
-write_ambiguous_var (struct block *block, char *name, int len)
+write_ambiguous_var (const struct block *block, char *name, int len)
 {
   struct symbol *sym =
     obstack_alloc (&temp_parse_space, sizeof (struct symbol));
   memset (sym, 0, sizeof (struct symbol));
   SYMBOL_DOMAIN (sym) = UNDEF_DOMAIN;
-  SYMBOL_LINKAGE_NAME (sym) = obsavestring (name, len, &temp_parse_space);
+  SYMBOL_LINKAGE_NAME (sym) = obstack_copy0 (&temp_parse_space, name, len);
   SYMBOL_LANGUAGE (sym) = language_ada;
 
   write_exp_elt_opcode (OP_VAR_VALUE);
@@ -2942,7 +2942,7 @@ get_symbol_field_type (struct symbol *sym, char *encoded_field_name)
    identifier).  */
 
 static struct type*
-write_var_or_type (struct block *block, struct stoken name0)
+write_var_or_type (const struct block *block, struct stoken name0)
 {
   int depth;
   char *encoded_name;
@@ -2953,7 +2953,7 @@ write_var_or_type (struct block *block, struct stoken name0)
 
   encoded_name = ada_encode (name0.ptr);
   name_len = strlen (encoded_name);
-  encoded_name = obsavestring (encoded_name, name_len, &temp_parse_space);
+  encoded_name = obstack_copy0 (&temp_parse_space, encoded_name, name_len);
   for (depth = 0; depth < MAX_RENAMING_CHAIN_LENGTH; depth += 1)
     {
       int tail_index;
@@ -2972,7 +2972,7 @@ write_var_or_type (struct block *block, struct stoken name0)
 
 	  encoded_name[tail_index] = '\0';
 	  nsyms = ada_lookup_symbol_list (encoded_name, block,
-					  VAR_DOMAIN, &syms, 1);
+					  VAR_DOMAIN, &syms);
 	  encoded_name[tail_index] = terminator;
 
 	  /* A single symbol may rename a package or object. */
@@ -3120,7 +3120,7 @@ write_name_assoc (struct stoken name)
     {
       struct ada_symbol_info *syms;
       int nsyms = ada_lookup_symbol_list (name.ptr, expression_context_block,
-					  VAR_DOMAIN, &syms, 1);
+					  VAR_DOMAIN, &syms);
       if (nsyms != 1 || SYMBOL_CLASS (syms[0].sym) == LOC_TYPEDEF)
 	write_exp_op_with_string (OP_NAME, name);
       else
