@@ -136,7 +136,7 @@
 
 
 /* Copy the first part of user declarations.  */
-#line 43 "f-exp.y"
+#line 42 "f-exp.y"
 
 
 #include "defs.h"
@@ -202,6 +202,12 @@
 #define yygindex f_yygindex
 #define yytable	 f_yytable
 #define yycheck	 f_yycheck
+#define yyss	f_yyss
+#define yysslim	f_yysslim
+#define yyssp	f_yyssp
+#define yystacksize f_yystacksize
+#define yyvs	f_yyvs
+#define yyvsp	f_yyvsp
 
 #ifndef YYDEBUG
 #define	YYDEBUG	1		/* Default to yydebug support */
@@ -235,7 +241,7 @@ static int match_string_literal (void);
 #endif
 
 #if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
-#line 132 "f-exp.y"
+#line 137 "f-exp.y"
 typedef union YYSTYPE {
     LONGEST lval;
     struct {
@@ -257,7 +263,7 @@ typedef union YYSTYPE {
     int *ivec;
   } YYSTYPE;
 /* Line 191 of yacc.c.  */
-#line 261 "f-exp.c"
+#line 267 "f-exp.c"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
@@ -266,14 +272,14 @@ typedef union YYSTYPE {
 
 
 /* Copy the second part of user declarations.  */
-#line 153 "f-exp.y"
+#line 158 "f-exp.y"
 
 /* YYSTYPE gets defined by %union */
 static int parse_number (char *, int, int, YYSTYPE *);
 
 
 /* Line 214 of yacc.c.  */
-#line 277 "f-exp.c"
+#line 283 "f-exp.c"
 
 #if ! defined (yyoverflow) || YYERROR_VERBOSE
 
@@ -482,15 +488,15 @@ static const yysigned_char yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const unsigned short yyrline[] =
 {
-       0,   232,   232,   233,   236,   242,   247,   251,   255,   259,
-     263,   267,   277,   276,   284,   287,   291,   295,   301,   307,
-     313,   319,   325,   329,   335,   341,   349,   353,   357,   361,
-     365,   369,   373,   377,   381,   385,   389,   393,   397,   401,
-     405,   409,   413,   417,   422,   426,   430,   436,   443,   452,
-     459,   462,   465,   473,   480,   488,   528,   531,   532,   575,
-     577,   579,   581,   583,   586,   588,   590,   594,   596,   601,
-     603,   605,   607,   609,   611,   613,   615,   617,   619,   621,
-     623,   625,   627,   632,   637,   644,   648
+       0,   237,   237,   238,   241,   247,   252,   256,   260,   264,
+     268,   272,   282,   281,   289,   292,   296,   300,   306,   312,
+     318,   324,   330,   334,   340,   346,   354,   358,   362,   366,
+     370,   374,   378,   382,   386,   390,   394,   398,   402,   406,
+     410,   414,   418,   422,   427,   431,   435,   441,   448,   457,
+     464,   467,   470,   478,   485,   493,   533,   536,   537,   580,
+     582,   584,   586,   588,   591,   593,   595,   599,   601,   606,
+     608,   610,   612,   614,   616,   618,   620,   622,   624,   626,
+     628,   630,   632,   637,   642,   649,   653
 };
 #endif
 
@@ -1361,237 +1367,237 @@ yyreduce:
   switch (yyn)
     {
         case 4:
-#line 237 "f-exp.y"
+#line 242 "f-exp.y"
     { write_exp_elt_opcode(OP_TYPE);
 			  write_exp_elt_type(yyvsp[0].tval);
 			  write_exp_elt_opcode(OP_TYPE); }
     break;
 
   case 5:
-#line 243 "f-exp.y"
+#line 248 "f-exp.y"
     { }
     break;
 
   case 6:
-#line 248 "f-exp.y"
+#line 253 "f-exp.y"
     { write_exp_elt_opcode (UNOP_IND); }
     break;
 
   case 7:
-#line 252 "f-exp.y"
+#line 257 "f-exp.y"
     { write_exp_elt_opcode (UNOP_ADDR); }
     break;
 
   case 8:
-#line 256 "f-exp.y"
+#line 261 "f-exp.y"
     { write_exp_elt_opcode (UNOP_NEG); }
     break;
 
   case 9:
-#line 260 "f-exp.y"
+#line 265 "f-exp.y"
     { write_exp_elt_opcode (UNOP_LOGICAL_NOT); }
     break;
 
   case 10:
-#line 264 "f-exp.y"
+#line 269 "f-exp.y"
     { write_exp_elt_opcode (UNOP_COMPLEMENT); }
     break;
 
   case 11:
-#line 268 "f-exp.y"
+#line 273 "f-exp.y"
     { write_exp_elt_opcode (UNOP_SIZEOF); }
     break;
 
   case 12:
-#line 277 "f-exp.y"
+#line 282 "f-exp.y"
     { start_arglist (); }
     break;
 
   case 13:
-#line 279 "f-exp.y"
+#line 284 "f-exp.y"
     { write_exp_elt_opcode (OP_F77_UNDETERMINED_ARGLIST);
 			  write_exp_elt_longcst ((LONGEST) end_arglist ());
 			  write_exp_elt_opcode (OP_F77_UNDETERMINED_ARGLIST); }
     break;
 
   case 15:
-#line 288 "f-exp.y"
+#line 293 "f-exp.y"
     { arglist_len = 1; }
     break;
 
   case 16:
-#line 292 "f-exp.y"
+#line 297 "f-exp.y"
     { arglist_len = 1; }
     break;
 
   case 17:
-#line 296 "f-exp.y"
+#line 301 "f-exp.y"
     { arglist_len++; }
     break;
 
   case 18:
-#line 302 "f-exp.y"
+#line 307 "f-exp.y"
     { write_exp_elt_opcode (OP_F90_RANGE); 
 			  write_exp_elt_longcst (NONE_BOUND_DEFAULT);
 			  write_exp_elt_opcode (OP_F90_RANGE); }
     break;
 
   case 19:
-#line 308 "f-exp.y"
+#line 313 "f-exp.y"
     { write_exp_elt_opcode (OP_F90_RANGE);
 			  write_exp_elt_longcst (HIGH_BOUND_DEFAULT);
 			  write_exp_elt_opcode (OP_F90_RANGE); }
     break;
 
   case 20:
-#line 314 "f-exp.y"
+#line 319 "f-exp.y"
     { write_exp_elt_opcode (OP_F90_RANGE);
 			  write_exp_elt_longcst (LOW_BOUND_DEFAULT);
 			  write_exp_elt_opcode (OP_F90_RANGE); }
     break;
 
   case 21:
-#line 320 "f-exp.y"
+#line 325 "f-exp.y"
     { write_exp_elt_opcode (OP_F90_RANGE);
 			  write_exp_elt_longcst (BOTH_BOUND_DEFAULT);
 			  write_exp_elt_opcode (OP_F90_RANGE); }
     break;
 
   case 22:
-#line 326 "f-exp.y"
+#line 331 "f-exp.y"
     { }
     break;
 
   case 23:
-#line 330 "f-exp.y"
+#line 335 "f-exp.y"
     { write_exp_elt_opcode(OP_COMPLEX);
 			  write_exp_elt_type (parse_f_type->builtin_complex_s16);
                 	  write_exp_elt_opcode(OP_COMPLEX); }
     break;
 
   case 24:
-#line 336 "f-exp.y"
+#line 341 "f-exp.y"
     { write_exp_elt_opcode (UNOP_CAST);
 			  write_exp_elt_type (yyvsp[-2].tval);
 			  write_exp_elt_opcode (UNOP_CAST); }
     break;
 
   case 25:
-#line 342 "f-exp.y"
+#line 347 "f-exp.y"
     { write_exp_elt_opcode (STRUCTOP_STRUCT);
                           write_exp_string (yyvsp[0].sval);
                           write_exp_elt_opcode (STRUCTOP_STRUCT); }
     break;
 
   case 26:
-#line 350 "f-exp.y"
+#line 355 "f-exp.y"
     { write_exp_elt_opcode (BINOP_REPEAT); }
     break;
 
   case 27:
-#line 354 "f-exp.y"
+#line 359 "f-exp.y"
     { write_exp_elt_opcode (BINOP_EXP); }
     break;
 
   case 28:
-#line 358 "f-exp.y"
+#line 363 "f-exp.y"
     { write_exp_elt_opcode (BINOP_MUL); }
     break;
 
   case 29:
-#line 362 "f-exp.y"
+#line 367 "f-exp.y"
     { write_exp_elt_opcode (BINOP_DIV); }
     break;
 
   case 30:
-#line 366 "f-exp.y"
+#line 371 "f-exp.y"
     { write_exp_elt_opcode (BINOP_ADD); }
     break;
 
   case 31:
-#line 370 "f-exp.y"
+#line 375 "f-exp.y"
     { write_exp_elt_opcode (BINOP_SUB); }
     break;
 
   case 32:
-#line 374 "f-exp.y"
+#line 379 "f-exp.y"
     { write_exp_elt_opcode (BINOP_LSH); }
     break;
 
   case 33:
-#line 378 "f-exp.y"
+#line 383 "f-exp.y"
     { write_exp_elt_opcode (BINOP_RSH); }
     break;
 
   case 34:
-#line 382 "f-exp.y"
+#line 387 "f-exp.y"
     { write_exp_elt_opcode (BINOP_EQUAL); }
     break;
 
   case 35:
-#line 386 "f-exp.y"
+#line 391 "f-exp.y"
     { write_exp_elt_opcode (BINOP_NOTEQUAL); }
     break;
 
   case 36:
-#line 390 "f-exp.y"
+#line 395 "f-exp.y"
     { write_exp_elt_opcode (BINOP_LEQ); }
     break;
 
   case 37:
-#line 394 "f-exp.y"
+#line 399 "f-exp.y"
     { write_exp_elt_opcode (BINOP_GEQ); }
     break;
 
   case 38:
-#line 398 "f-exp.y"
+#line 403 "f-exp.y"
     { write_exp_elt_opcode (BINOP_LESS); }
     break;
 
   case 39:
-#line 402 "f-exp.y"
+#line 407 "f-exp.y"
     { write_exp_elt_opcode (BINOP_GTR); }
     break;
 
   case 40:
-#line 406 "f-exp.y"
+#line 411 "f-exp.y"
     { write_exp_elt_opcode (BINOP_BITWISE_AND); }
     break;
 
   case 41:
-#line 410 "f-exp.y"
+#line 415 "f-exp.y"
     { write_exp_elt_opcode (BINOP_BITWISE_XOR); }
     break;
 
   case 42:
-#line 414 "f-exp.y"
+#line 419 "f-exp.y"
     { write_exp_elt_opcode (BINOP_BITWISE_IOR); }
     break;
 
   case 43:
-#line 418 "f-exp.y"
+#line 423 "f-exp.y"
     { write_exp_elt_opcode (BINOP_LOGICAL_AND); }
     break;
 
   case 44:
-#line 423 "f-exp.y"
+#line 428 "f-exp.y"
     { write_exp_elt_opcode (BINOP_LOGICAL_OR); }
     break;
 
   case 45:
-#line 427 "f-exp.y"
+#line 432 "f-exp.y"
     { write_exp_elt_opcode (BINOP_ASSIGN); }
     break;
 
   case 46:
-#line 431 "f-exp.y"
+#line 436 "f-exp.y"
     { write_exp_elt_opcode (BINOP_ASSIGN_MODIFY);
 			  write_exp_elt_opcode (yyvsp[-1].opcode);
 			  write_exp_elt_opcode (BINOP_ASSIGN_MODIFY); }
     break;
 
   case 47:
-#line 437 "f-exp.y"
+#line 442 "f-exp.y"
     { write_exp_elt_opcode (OP_LONG);
 			  write_exp_elt_type (yyvsp[0].typed_val.type);
 			  write_exp_elt_longcst ((LONGEST)(yyvsp[0].typed_val.val));
@@ -1599,7 +1605,7 @@ yyreduce:
     break;
 
   case 48:
-#line 444 "f-exp.y"
+#line 449 "f-exp.y"
     { YYSTYPE val;
 			  parse_number (yyvsp[0].ssym.stoken.ptr, yyvsp[0].ssym.stoken.length, 0, &val);
 			  write_exp_elt_opcode (OP_LONG);
@@ -1609,7 +1615,7 @@ yyreduce:
     break;
 
   case 49:
-#line 453 "f-exp.y"
+#line 458 "f-exp.y"
     { write_exp_elt_opcode (OP_DOUBLE);
 			  write_exp_elt_type (parse_f_type->builtin_real_s8);
 			  write_exp_elt_dblcst (yyvsp[0].dval);
@@ -1617,7 +1623,7 @@ yyreduce:
     break;
 
   case 52:
-#line 466 "f-exp.y"
+#line 471 "f-exp.y"
     { write_exp_elt_opcode (OP_LONG);
 			  write_exp_elt_type (parse_f_type->builtin_integer);
 			  CHECK_TYPEDEF (yyvsp[-1].tval);
@@ -1626,7 +1632,7 @@ yyreduce:
     break;
 
   case 53:
-#line 474 "f-exp.y"
+#line 479 "f-exp.y"
     { write_exp_elt_opcode (OP_BOOL);
 			  write_exp_elt_longcst ((LONGEST) yyvsp[0].lval);
 			  write_exp_elt_opcode (OP_BOOL);
@@ -1634,7 +1640,7 @@ yyreduce:
     break;
 
   case 54:
-#line 481 "f-exp.y"
+#line 486 "f-exp.y"
     {
 			  write_exp_elt_opcode (OP_STRING);
 			  write_exp_string (yyvsp[0].sval);
@@ -1643,7 +1649,7 @@ yyreduce:
     break;
 
   case 55:
-#line 489 "f-exp.y"
+#line 494 "f-exp.y"
     { struct symbol *sym = yyvsp[0].ssym.sym;
 
 			  if (sym)
@@ -1683,7 +1689,7 @@ yyreduce:
     break;
 
   case 58:
-#line 533 "f-exp.y"
+#line 538 "f-exp.y"
     {
 		  /* This is where the interesting stuff happens.  */
 		  int done = 0;
@@ -1727,122 +1733,122 @@ yyreduce:
     break;
 
   case 59:
-#line 576 "f-exp.y"
+#line 581 "f-exp.y"
     { push_type (tp_pointer); yyval.voidval = 0; }
     break;
 
   case 60:
-#line 578 "f-exp.y"
+#line 583 "f-exp.y"
     { push_type (tp_pointer); yyval.voidval = yyvsp[0].voidval; }
     break;
 
   case 61:
-#line 580 "f-exp.y"
+#line 585 "f-exp.y"
     { push_type (tp_reference); yyval.voidval = 0; }
     break;
 
   case 62:
-#line 582 "f-exp.y"
+#line 587 "f-exp.y"
     { push_type (tp_reference); yyval.voidval = yyvsp[0].voidval; }
     break;
 
   case 64:
-#line 587 "f-exp.y"
+#line 592 "f-exp.y"
     { yyval.voidval = yyvsp[-1].voidval; }
     break;
 
   case 65:
-#line 589 "f-exp.y"
+#line 594 "f-exp.y"
     { push_type (tp_function); }
     break;
 
   case 66:
-#line 591 "f-exp.y"
+#line 596 "f-exp.y"
     { push_type (tp_function); }
     break;
 
   case 67:
-#line 595 "f-exp.y"
+#line 600 "f-exp.y"
     { yyval.voidval = 0; }
     break;
 
   case 68:
-#line 597 "f-exp.y"
+#line 602 "f-exp.y"
     { xfree (yyvsp[-1].tvec); yyval.voidval = 0; }
     break;
 
   case 69:
-#line 602 "f-exp.y"
+#line 607 "f-exp.y"
     { yyval.tval = yyvsp[0].tsym.type; }
     break;
 
   case 70:
-#line 604 "f-exp.y"
+#line 609 "f-exp.y"
     { yyval.tval = parse_f_type->builtin_integer; }
     break;
 
   case 71:
-#line 606 "f-exp.y"
+#line 611 "f-exp.y"
     { yyval.tval = parse_f_type->builtin_integer_s2; }
     break;
 
   case 72:
-#line 608 "f-exp.y"
+#line 613 "f-exp.y"
     { yyval.tval = parse_f_type->builtin_character; }
     break;
 
   case 73:
-#line 610 "f-exp.y"
+#line 615 "f-exp.y"
     { yyval.tval = parse_f_type->builtin_logical_s8; }
     break;
 
   case 74:
-#line 612 "f-exp.y"
+#line 617 "f-exp.y"
     { yyval.tval = parse_f_type->builtin_logical; }
     break;
 
   case 75:
-#line 614 "f-exp.y"
+#line 619 "f-exp.y"
     { yyval.tval = parse_f_type->builtin_logical_s2; }
     break;
 
   case 76:
-#line 616 "f-exp.y"
+#line 621 "f-exp.y"
     { yyval.tval = parse_f_type->builtin_logical_s1; }
     break;
 
   case 77:
-#line 618 "f-exp.y"
+#line 623 "f-exp.y"
     { yyval.tval = parse_f_type->builtin_real; }
     break;
 
   case 78:
-#line 620 "f-exp.y"
+#line 625 "f-exp.y"
     { yyval.tval = parse_f_type->builtin_real_s8; }
     break;
 
   case 79:
-#line 622 "f-exp.y"
+#line 627 "f-exp.y"
     { yyval.tval = parse_f_type->builtin_real_s16; }
     break;
 
   case 80:
-#line 624 "f-exp.y"
+#line 629 "f-exp.y"
     { yyval.tval = parse_f_type->builtin_complex_s8; }
     break;
 
   case 81:
-#line 626 "f-exp.y"
+#line 631 "f-exp.y"
     { yyval.tval = parse_f_type->builtin_complex_s16; }
     break;
 
   case 82:
-#line 628 "f-exp.y"
+#line 633 "f-exp.y"
     { yyval.tval = parse_f_type->builtin_complex_s32; }
     break;
 
   case 83:
-#line 633 "f-exp.y"
+#line 638 "f-exp.y"
     { yyval.tvec = (struct type **) xmalloc (sizeof (struct type *) * 2);
 		  yyval.ivec[0] = 1;	/* Number of types in vector */
 		  yyval.tvec[1] = yyvsp[0].tval;
@@ -1850,7 +1856,7 @@ yyreduce:
     break;
 
   case 84:
-#line 638 "f-exp.y"
+#line 643 "f-exp.y"
     { int len = sizeof (struct type *) * (++(yyvsp[-2].ivec[0]) + 1);
 		  yyval.tvec = (struct type **) xrealloc ((char *) yyvsp[-2].tvec, len);
 		  yyval.tvec[yyval.ivec[0]] = yyvsp[0].tval;
@@ -1858,7 +1864,7 @@ yyreduce:
     break;
 
   case 85:
-#line 645 "f-exp.y"
+#line 650 "f-exp.y"
     {  yyval.sval = yyvsp[0].ssym.stoken; }
     break;
 
@@ -1866,7 +1872,7 @@ yyreduce:
     }
 
 /* Line 1000 of yacc.c.  */
-#line 1870 "f-exp.c"
+#line 1876 "f-exp.c"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -2091,7 +2097,7 @@ yyreturn:
 }
 
 
-#line 658 "f-exp.y"
+#line 663 "f-exp.y"
 
 
 /* Take care of parsing a number (anything that starts with a digit).
@@ -2101,11 +2107,7 @@ yyreturn:
 /*** Needs some error checking for the float case ***/
 
 static int
-parse_number (p, len, parsed_float, putithere)
-     char *p;
-     int len;
-     int parsed_float;
-     YYSTYPE *putithere;
+parse_number (char *p, int len, int parsed_float, YYSTYPE *putithere)
 {
   LONGEST n = 0;
   LONGEST prevn = 0;
@@ -2332,8 +2334,7 @@ static int tempbufindex;	/* Current index into buffer */
    first one on demand.  */
 
 static void
-growbuf_by_size (count)
-     int count;
+growbuf_by_size (int count)
 {
   int growby;
 
@@ -2611,9 +2612,13 @@ yylex (void)
   {
     char *tmp = copy_name (yylval.sval);
     struct symbol *sym;
-    int is_a_field_of_this = 0;
+    struct field_of_this_result is_a_field_of_this;
     int hextype;
     
+    /* Initialize this in case we *don't* use it in this call; that
+       way we can refer to it unconditionally below.  */
+    memset (&is_a_field_of_this, 0, sizeof (is_a_field_of_this));
+
     sym = lookup_symbol (tmp, expression_context_block,
 			 VAR_DOMAIN,
 			 parse_language->la_language == language_cplus
@@ -2641,21 +2646,20 @@ yylex (void)
 	if (hextype == INT)
 	  {
 	    yylval.ssym.sym = sym;
-	    yylval.ssym.is_a_field_of_this = is_a_field_of_this;
+	    yylval.ssym.is_a_field_of_this = is_a_field_of_this.type != NULL;
 	    return NAME_OR_INT;
 	  }
       }
     
     /* Any other kind of symbol */
     yylval.ssym.sym = sym;
-    yylval.ssym.is_a_field_of_this = is_a_field_of_this;
+    yylval.ssym.is_a_field_of_this = is_a_field_of_this.type != NULL;
     return NAME;
   }
 }
 
 void
-yyerror (msg)
-     char *msg;
+yyerror (char *msg)
 {
   if (prev_lexptr)
     lexptr = prev_lexptr;
